/*
 * modbus_tiny_functions.c
 *
 *      Author: arkhnchul
 */

#include <stdint.h>
#include "modbus_tiny.h"

typedef enum {
    MB_REG_FOOPARAM = 0, MB_REG_BARPARAM = 1
} mb_register_name;

typedef enum {
    MB_COIL_FOOBIT = 0, MB_COIL_BARBIT = 1
} mb_coil_name;

/*
 * Device-specific register read actions
 * */
#if defined(MB_HAS_FUNCTION_READ_HOLDING_REGISTERS) || defined(MB_HAS_FUNCTION_READ_INPUT_REGISTERS)
uint16_t mb_port_read_register(uint16_t register_addr) {
    switch (register_addr) {
    // TODO: do something sane with device "registers"
    case MB_REG_FOOPARAM:
        return 11;
    case MB_REG_BARPARAM:
        return 22;
    default:
        mb_raise_exception(MB_EXCEPTION_WRONG_ADDRESS);
        return 0;
    }
}
#endif

/*
 * Device-specific register write actions
 * */
#ifdef MB_HAS_FUNCTION_WRITE_SINGLE_REGISTER
void mb_port_write_register(uint16_t register_addr, uint16_t value) {
    // TODO: remove this line
    value = register_addr; // just make compiler not to cry about unused "value" variable during tests

    // TODO: do something sane with device "registers"
    switch (register_addr) {
    default: /* all registers are read-only, unless there are write action specified above
	 for specific address */
        mb_raise_exception(MB_EXCEPTION_WRONG_ADDRESS);
    }
}
#endif

/*
 * Device-specific coil read actions
 * */
#if defined(MB_HAS_FUNCTION_READ_COILS) || defined(MB_HAS_FUNCTION_READ_DISCRETE_INPUTS)
uint8_t mb_port_read_coil(uint16_t coil_addr) {
    // TODO: do something sane with device "coils"
    switch(coil_addr) {
    case MB_COIL_FOOBIT:
        return 1;
    case MB_COIL_BARBIT:
        return 0;
    default:
        mb_raise_exception(MB_EXCEPTION_WRONG_ADDRESS);
        return 0;
    }
}
#endif

/*
 * Device-specific coil write actions
 * */
#ifdef MB_HAS_FUNCTION_WRITE_SINGLE_COIL
void mb_port_write_coil(uint16_t coil_addr, uint8_t value) {
    // TODO: remove this line
    value = coil_addr; // just make compiler not to cry about unused "value" variable during tests

    // TODO: do something sane with device "coils"
    switch (coil_addr) {
    default: /* all coils are read-only, unless there are write action specified above
	 for specific address */
        mb_raise_exception(MB_EXCEPTION_WRONG_ADDRESS);
    }
}
#endif
